@@ -34,6 +34,11 @@ from relatorio.templates.opendocument import Template, GENSHI_EXPR,\
     GENSHI_URI, RELATORIO_URI, fod2od, remove_node_keeping_tail, \
     escape_xml_invalid_chars
 
+try:
+    unicode
+except NameError:
+    unicode = str
+
 OO_TABLE_NS = "urn:oasis:names:tc:opendocument:xmlns:table:1.0"
 
 
@@ -104,6 +109,93 @@ class TestOOTemplating(unittest.TestCase):
             child.get('{http://genshi.edgewall.org/}replace'),
             '__relatorio_escape_invalid_chars(foo)')
 
+    @unittest.skipIf(
+        not hasattr(unittest.TestCase, 'assertRegex'),
+        'assertRegex not supported')
+    def test_directives_in_cell(self):
+        "Testing the guess type of directive inside cell"
+        xml = b'''<xml xmlns:text="urn:text" xmlns:table="urn:table" xmlns:xlink="urn:xlink">
+                    <table:table-cell>
+                        <text:p>
+                            <text:a xlink:href="relatorio://foo">foo</text:a>
+                        </text:p>
+                    </table:table-cell>
+                 </xml>'''
+        interpolated = self.oot.insert_directives(xml)
+        root_interpolated = lxml.etree.parse(interpolated).getroot()
+        cell = root_interpolated[0]
+        self.assertRegex(
+            cell.get('{http://genshi.edgewall.org/}attrs'),
+            r'__relatorio_guess_type\(__relatorio_store_cache\([0-9]*, foo\)\)'
+            )
+        child = cell[0][0]
+        self.assertRegex(
+            child.get('{http://genshi.edgewall.org/}replace'),
+            r'__relatorio_get_cache\([0-9]*\)')
+
+    def test_directives_in_cell_with_children(self):
+        "Testing directive inside cell with children"
+        xml = b'''<xml xmlns:text="urn:text" xmlns:table="urn:table" xmlns:xlink="urn:xlink">
+                    <table:table-cell>
+                        <text:p>
+                            <text:a xlink:href="relatorio://foo">foo</text:a>
+                        </text:p>
+                        <text:p>bar</text:p>
+                    </table:table-cell>
+                 </xml>'''
+        interpolated = self.oot.insert_directives(xml)
+        root_interpolated = lxml.etree.parse(interpolated).getroot()
+        cell = root_interpolated[0]
+        self.assertEqual(len(cell), 2)
+        self.assertFalse(
+            cell.get('{http://genshi.edgewall.org/}attrs'))
+        child = cell[0][0]
+        self.assertEqual(
+            child.get('{http://genshi.edgewall.org/}replace'),
+            '__relatorio_escape_invalid_chars(foo)')
+
+    def test_directives_in_cell_with_text(self):
+        "Testing directive inside cell with text"
+        xml = b'''<xml xmlns:text="urn:text" xmlns:table="urn:table" xmlns:xlink="urn:xlink">
+                    <table:table-cell>
+                        <text:p>
+                            bar
+                            <text:a xlink:href="relatorio://foo">foo</text:a>
+                        </text:p>
+                    </table:table-cell>
+                 </xml>'''
+        interpolated = self.oot.insert_directives(xml)
+        root_interpolated = lxml.etree.parse(interpolated).getroot()
+        cell = root_interpolated[0]
+        self.assertEqual(len(cell), 1)
+        self.assertFalse(
+            cell.get('{http://genshi.edgewall.org/}attrs'))
+        child = cell[0][0]
+        self.assertEqual(
+            child.get('{http://genshi.edgewall.org/}replace'),
+            '__relatorio_escape_invalid_chars(foo)')
+
+    def test_directives_in_cell_with_tail(self):
+        "Testing directive inside cell with text"
+        xml = b'''<xml xmlns:text="urn:text" xmlns:table="urn:table" xmlns:xlink="urn:xlink">
+                    <table:table-cell>
+                        <text:p>
+                            <text:a xlink:href="relatorio://foo">foo</text:a>
+                            bar
+                        </text:p>
+                    </table:table-cell>
+                 </xml>'''
+        interpolated = self.oot.insert_directives(xml)
+        root_interpolated = lxml.etree.parse(interpolated).getroot()
+        cell = root_interpolated[0]
+        self.assertEqual(len(cell), 1)
+        self.assertFalse(
+            cell.get('{http://genshi.edgewall.org/}attrs'))
+        child = cell[0][0]
+        self.assertEqual(
+            child.get('{http://genshi.edgewall.org/}replace'),
+            '__relatorio_escape_invalid_chars(foo)')
+
     def test_column_looping(self):
         xml = b'''
 <table:table
@@ -206,7 +298,7 @@ class TestOOTemplating(unittest.TestCase):
         <table:covered-table-cell/>
         <table:covered-table-cell/>
     </table:table-row>
-</table:table>'''
+</table:table>'''  # noqa: E501
         interpolated = self.oot.insert_directives(xml)
         root = lxml.etree.parse(interpolated).getroot()
         child2 = root[1]
@@ -250,7 +342,7 @@ class TestOOTemplating(unittest.TestCase):
         "Test warning for missing statement text"
         xml = b'''<xml xmlns:text="urn:text" xmlns:xlink="urn:xlink">
                     <text:a xlink:href="relatorio://content:foo">content:bar</text:a>
-                  </xml>'''
+                  </xml>'''  # noqa: E501
 
         with self.assertWarnsRegex(
                 UserWarning,
@@ -266,7 +358,7 @@ class TestOOTemplating(unittest.TestCase):
         xml = b'''<xml xmlns:text="urn:text" xmlns:xlink="urn:xlink">
                     <text:a xlink:href="relatorio://with foo='test'">with foo='test'</text:a>
                     <text:a xlink:href="relatorio:///with"></text:a>
-                  </xml>'''
+                  </xml>'''  # noqa: E501
 
         with self.assertWarnsRegex(
                 UserWarning,
@@ -282,7 +374,7 @@ class TestOOTemplating(unittest.TestCase):
                     zzz
                     <text:a xlink:href="relatorio:///if">/if</text:a>
                     aaa
-                 </xml>'''
+                 </xml>'''  # noqa: E501
         interpolated = self.oot.insert_directives(xml)
         root_interpolated = lxml.etree.parse(interpolated).getroot()
         child = root_interpolated[0]
@@ -333,22 +425,24 @@ class TestOOTemplating(unittest.TestCase):
         tree = lxml.etree.parse(StringIO(rendered[25:styles_idx]))
         root = tree.getroot()
         images = root.xpath('//draw:frame', namespaces=self.oot.namespaces)
-        self.assertEqual(len(images), 3)
+        self.assertEqual(len(images), 4)
         self.assertFalse(
             images[0].get('{%s}name' % self.oot.namespaces['draw']))
+        self.assertFalse(
+            images[1].get('{%s}name' % self.oot.namespaces['draw']))
         self.assertEqual(
-            images[1].get('{%s}name' % self.oot.namespaces['draw']), 'One')
+            images[2].get('{%s}name' % self.oot.namespaces['draw']), 'One')
         self.assertEqual(
-            images[2].get('{%s}name' % self.oot.namespaces['draw']), 'Two')
+            images[3].get('{%s}name' % self.oot.namespaces['draw']), 'Two')
         self.assertEqual(
-            images[1].get('{%s}width' % self.oot.namespaces['svg']), '1.732cm')
-        self.assertEqual(
-            images[1].get('{%s}height' % self.oot.namespaces['svg']),
-            '1.513cm')
-        self.assertEqual(
-            images[2].get('{%s}width' % self.oot.namespaces['svg']), '2cm')
+            images[2].get('{%s}width' % self.oot.namespaces['svg']), '1.732cm')
         self.assertEqual(
             images[2].get('{%s}height' % self.oot.namespaces['svg']),
+            '1.513cm')
+        self.assertEqual(
+            images[3].get('{%s}width' % self.oot.namespaces['svg']), '2cm')
+        self.assertEqual(
+            images[3].get('{%s}height' % self.oot.namespaces['svg']),
             '2.2cm')
 
     def test_regexp(self):
